/*
** SSYNC: Simple minded filesystem sync utility
** Copyright (C) 2000 Michael W. Shaffer <mwshaffer@yahoo.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.  
**
** You should have received a copy of the GNU General Public License
** along with this program (see the file COPYING). If not, write to:
**
** The Free Software Foundation, Inc.
** 59 Temple Place, Suite 330,
** Boston, MA  02111-1307  USA
*/

#ifndef __SSYNC_H__
#define __SSYNC_H__

#define SSYNC_VERSION "1.7"

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <utime.h>
#include "hash.h"
#include "log.h"

struct flags {
	int no_sync_data;
	int no_sync_time;
	int no_sync_meta;
	int update_only;
	int test;
};

extern struct flags flags;

struct stats {
	unsigned long dirs;
	unsigned long files;
	unsigned long links;
	unsigned long dirs_updated;
	unsigned long regs_updated;
	unsigned long lnks_updated;
	unsigned long deleted;
	unsigned long bytes;
	unsigned long kbytes;
	unsigned long mbytes;
	unsigned long gbytes;
	unsigned long tbytes;
	unsigned long unknowns;
	unsigned long errors;
};

extern struct stats stats;

extern struct hash_table work;
extern struct hash_table dups;

void ssync_shutdown (void);
int ssync_startup (int argc, char **argv);
void get_bytes (char *buf, unsigned long bufsize);
void recursive_unlink (char *path);
void process_dir (char *src, struct stat *s, char *dst, struct stat *d);
void process_reg (char *src, struct stat *s, char *dst, struct stat *d);
void process_lnk (char *src, struct stat *s, char *dst, struct stat *d);

extern inline
void check_type (char *src, struct stat *s, char *dst, struct stat *d)
{
	int fd = -1;
	char msg[256];
	struct utimbuf u;

	if ((flags.test || !(src && s && dst && d)) ||
		((S_IFMT & s->st_mode) == (S_IFMT & d->st_mode)) ||
		(flags.update_only && !(s->st_mtime > d->st_mtime))) {
		goto EXIT;
	}

	recursive_unlink (dst);
	memset (&u, 0, sizeof (struct utimbuf));

	if (S_ISDIR (s->st_mode)) {
		if (mkdir (dst, 0700) < 0) {
			memset (msg, 0, sizeof (msg));
			snprintf (msg, (sizeof (msg) - 1),
				"check_type: mkdir() on node '%s' failed: %s",
				dst, strerror (errno));
			LOG (warn, msg);
			stats.errors++;
			goto EXIT;
		}

		if (utime (dst, &u) < 0) {
			memset (msg, 0, sizeof (msg));
			snprintf (msg, (sizeof (msg) - 1),
				"check_type: utime() on node '%s' failed: %s",
				dst, strerror (errno));
			LOG (warn, msg);
			stats.errors++;
			goto EXIT;
		}
	} else if (S_ISREG (s->st_mode)) {
		if ((fd = open (dst, O_CREAT|O_WRONLY|O_TRUNC, 0600)) < 0) {
			memset (msg, 0, sizeof (msg));
			snprintf (msg, (sizeof (msg) - 1),
				"check_type: open() on node '%s' failed: %s",
				dst, strerror (errno));
			LOG (warn, msg);
			stats.errors++;
			goto EXIT;
		}
		close (fd);

		if (utime (dst, &u) < 0) {
			memset (msg, 0, sizeof (msg));
			snprintf (msg, (sizeof (msg) - 1),
				"check_type: utime() on node '%s' failed: %s",
				dst, strerror (errno));
			LOG (warn, msg);
			stats.errors++;
			goto EXIT;
		}
	} else if (S_ISLNK (s->st_mode)) {
		if (symlink (".", dst) < 0) {
			memset (msg, 0, sizeof (msg));
			snprintf (msg, (sizeof (msg) - 1),
				"check_type: symlink() on node '%s' failed: %s",
				dst, strerror (errno));
			LOG (warn, msg);
			stats.errors++;
			goto EXIT;
		}
	} else {
		goto EXIT;
	}


	lstat (dst, d);

EXIT:
	return;
}

extern inline
void process (char *src, struct stat *s, char *dst, struct stat *d)
{
	char msg[256];

	if (!(src && s && dst && d))
		goto EXIT;

	if (S_ISDIR (s->st_mode)) {
		check_type (src, s, dst, d);
		process_dir (src, s, dst, d); 
	} else if (S_ISREG (s->st_mode)) {
		check_type (src, s, dst, d);
		process_reg (src, s, dst, d); 
	} else if (S_ISLNK (s->st_mode)) {
		check_type (src, s, dst, d);
		process_lnk (src, s, dst, d); 
	} else {
		memset (msg, 0, sizeof (msg));
		snprintf (msg, (sizeof (msg) - 1),
			"skipping node '%s' of type 0x%08x",
			src, (S_IFMT & s->st_mode));
		LOG (warn, msg); 
		stats.unknowns++;
	}

EXIT:
	return;
}

#endif /* __SSYNC_H__ */

